"use strict";
/*
This file is part of uvcc -- USB Video Class (UVC) device configurator.
Copyright (C) 2018, 2019, 2020 Joel Purra <https://joelpurra.com/>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const bluebird_1 = __importDefault(require("bluebird"));
class CameraHelper {
    constructor(output, cameraControlHelper, camera) {
        this.output = output;
        this.cameraControlHelper = cameraControlHelper;
        this.camera = camera;
        assert_1.default.strictEqual(arguments.length, 3);
        assert_1.default(typeof this.output === "object");
        assert_1.default(typeof this.cameraControlHelper === "object");
        assert_1.default(typeof this.camera === "object");
    }
    async getValues(controlName) {
        const gettableControlNames = await this.cameraControlHelper.getGettableControlNames();
        if (!gettableControlNames.includes(controlName)) {
            throw new Error(`Could not find a gettable control named ${JSON.stringify(controlName)}.`);
        }
        const valueObject = await this.camera.get(controlName);
        return valueObject;
    }
    async getRange(controlName) {
        const rangedControlNames = await this.cameraControlHelper.getRangedControlNames();
        if (!rangedControlNames.includes(controlName)) {
            throw new Error(`Could not find a ranged control named ${JSON.stringify(controlName)}.`);
        }
        const range = this.camera.range(controlName);
        return range;
    }
    async setValues(controlName, ...values) {
        const settableControlNames = await this.cameraControlHelper.getSettableControlNames();
        if (!settableControlNames.includes(controlName)) {
            throw new Error(`Could not find a settable control named ${JSON.stringify(controlName)}.`);
        }
        await this.camera.set(controlName, ...values);
        return undefined;
    }
    async getControlNames() {
        return this.cameraControlHelper.getControlNames();
    }
    async getRanges() {
        // TODO: replace with Object.fromEntries(...);
        return bluebird_1.default.reduce(this.cameraControlHelper.getRangedControlNames(), 
        // eslint-disable-next-line @typescript-eslint/prefer-readonly-parameter-types
        async (object, controlName) => {
            try {
                object[controlName] = await this.getRange(controlName);
            }
            catch (error) {
                // TODO: ignore only specific errors, such as usb.LIBUSB_TRANSFER_STALL?
                this.output.verbose("Error getting range, ignoring.", controlName, error);
            }
            return object;
        }, {});
    }
    async getSettableControls() {
        // TODO: replace with Object.fromEntries(...);
        return bluebird_1.default.reduce(this.cameraControlHelper.getSettableControlNames(), 
        // eslint-disable-next-line @typescript-eslint/prefer-readonly-parameter-types
        async (object, controlName) => {
            try {
                object[controlName] = await this.getValues(controlName);
            }
            catch (error) {
                // TODO: ignore only specific errors, such as usb.LIBUSB_TRANSFER_STALL?
                this.output.verbose("Error getting settable value, ignoring.", controlName, error);
            }
            return object;
        }, {});
    }
    async setControls(configuration) {
        const controlNames = Object.keys(configuration);
        const settableControlNames = await this.cameraControlHelper.getSettableControlNames();
        // NOTE: checking all control names before attempting to set any.
        const nonSettableNames = controlNames.filter((controlName) => !settableControlNames.includes(controlName));
        if (nonSettableNames.length !== 0) {
            throw new Error(`Could not find a settable controls, aborting setting values: ${JSON.stringify(nonSettableNames)}`);
        }
        await bluebird_1.default.map(
        // eslint-disable-next-line unicorn/no-fn-reference-in-iterator
        controlNames, async (controlName) => {
            const value = configuration[controlName];
            try {
                await this.setValues(controlName, value);
            }
            catch (error) {
                // TODO: ignore only specific errors, such as usb.LIBUSB_TRANSFER_STALL?
                this.output.verbose("Error setting value, ignoring.", controlName, value, error);
            }
        });
    }
}
exports.default = CameraHelper;
